<?php

// SessionToken.class.php - Class for manipulating session-based authentication tokens
//
// $Id: SessionToken.class.php 1424 2008-12-29 16:09:36Z tim $
//
// (c)2001-2004 Tim Jackson (tim@timj.co.uk)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

class Toolkit_SessionTokenHandler
{
	
	var $SessionVarName = 'tokens';
	var $ExpungeEnabled = false;
	
	// If valid_time is 0, token is valid till end of session
	function generate ($action='any', $valid_time = 3600) {
		$Token = new SessionToken;
		$Token->action = $action;
		$Token->id = uniqid(rand());
		$Token->timestamp = time();
		$Token->valid_time = $valid_time;
		$_SESSION[$this->SessionVarName][] = $Token;
		return $Token->id;
	}
	
	function validate ($TokenID, $action='any') {
		if ($this->ExpungeEnabled) $this->expungeExpired();
		
		foreach ($_SESSION[$this->SessionVarName] as $ID => $Token) {
			if ($Token->id == $TokenID && $Token->action == $action) {		// If token is valid,
				unset($_SESSION[$this->SessionVarName][$ID]);	// delete it so it can't be used again
				return true;
			}
		}
		
		// Fail
		return false;
	}
	
	// Check if this token is expired
	function expungeExpired () {
		foreach ($_SESSION[$this->SessionVarName] as $ID => $Token) {
			if ($Token->valid_time > 0) {
				if ($Token->timestamp + $Token->valid_time < time()) {
					unset($_SESSION[$ID]);
				}
			}
		}
	}
}

class SessionToken
{
	var $id; // unique, hard-to-guess ID
	var $action;	// Action that token is valid for
	var $timestamp;	// UNIX timestamp when token was created
	var $valid_time;	// Length of time (in seconds) that the token is valid for (0=unlimited)
}
