<?php

// DB_adodb.class.php - abstract database functions
// Generic ADODB version

// $Id: DB_adodb.class.php 2194 2010-09-10 10:59:24Z tim $
//
// (c)2003-2008 Tim Jackson (tim@timj.co.uk)
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

global $PHPOF_ADODB_INCLUDE_PATH;

if (ini_get('register_globals')) {
	trigger_error('Do not use PHPOF with register_globals turned on', E_USER_ERROR);
}

require_once('adodb.php');
require_once($PHPOF_ADODB_INCLUDE_PATH.'adodb.inc.php');

/**
 * ADODB back-end driver.
 * See the MySQL driver for code documentation.
 *
 */
class DB_adodb {

	var $_dbh;					// Database handle
	var $_dbtype;				// Database type
	var $_dbhost;				// Database host
	var $_dbname;				// Database name
	var $_adodb;

	// Connect to the database server and select the database we wish to use
	function DB_adodb ($dbhost, $username, $password, $dbname, $dbtype) {
		$this->_adodb = ADONewConnection($dbtype);
		$this->_adodb->Connect($dbhost, $username, $password, $dbname);
		
		// Required so that Resultset::FetchRow() works as expected
		$this->_adodb->SetFetchMode(ADODB_FETCH_ASSOC); 
		
		$this->_dbname = $dbname;
		$this->_dbhost = $dbhost;
		$this->_dbtype = $dbtype;
	}

	// Query the database
	function query ($query) {
		global $PHPOF_DEBUG_MODE;
		if ($PHPOF_DEBUG_MODE) phpof_debug("DB_adodb::query(): $query\n");
		
		// TODO: bit of a hack for MSSQL but this should at least stop queries failing
		$query = str_replace("\'","''",$query);
		
		$result = $this->_adodb->Execute($query);
		
		if ($result === false) {
			trigger_error("DB_adodb::query(): ADODB returned false (SQL query was: $query)", E_USER_ERROR);
		}
		
		$resobj = new DB_result($this->_adodb, $result);
		return $resobj;
	}
	
	// Perform a query, returning results from a particular page
	function query_paged ($query, $results_per_page, $page) {
		global $PHPOF_DEBUG_MODE;
		if ($page < 1) trigger_error("DB::query_paged passed a page number ($page) less than 1", E_USER_ERROR);
		$base = $results_per_page * ($page - 1);
		
		if ($PHPOF_DEBUG_MODE) phpof_debug("DB_adodb::query_paged(): $query\n");
		
		$result = $this->_adodb->SelectLimit($query, $results_per_page, $base);
		
		if ($result === false) {
			$err = "DB_adodb::query_paged(): ADODB returned false (SQL query was: $query with base $base, results $results_per_page)";
			trigger_error($err, E_USER_ERROR);
		}
		
		$resobj = new DB_result($this->_adodb, $result);
		$resobj->pages = $this->query_num_pages($query, $results_per_page);
		
		return $resobj;
	}
	
	// Get the number of pages that would be returned for a given query
	// TODO: optimise?
	function query_num_pages ($query, $results_per_page) {
		global $PHPOF_DEBUG_MODE;
		if ($results_per_page == 0) trigger_error('DB_adodb::query_num_pages() called with zero results per page',E_USER_ERROR);
		
		if ($PHPOF_DEBUG_MODE) phpof_debug("DB_adodb::query_num_pages(): $query\n");
		
		$result = $this->_adodb->Execute($query);
		
		if ($result === false) {
			trigger_error("DB_adodb::query_num_pages(): ADODB returned false (SQL query was: $query)", E_USER_ERROR);
		}
		
		$rows = $result->RecordCount();
		
		return ceil ($rows / $results_per_page);
	}
	
	function beginTransaction()
	{
		$this->_adodb->BeginTrans();
	}
	
	function rollbackTransaction()
	{
		$this->_adodb->RollbackTrans();
	}
	
	function commitTransaction()
	{
		$this->_adodb->CommitTrans();
	}
	
	function qstr($String)
	{
		// The underlying ADODB qstr() function returns a *quoted* string,
		// not just an *escaped* string so we have to strip off the quotes
		// (NB the PHPOF function qstr() ought to be renamed estr())
		return substr($this->_adodb->qstr($String),1,strlen($String));
	}
}

class DB_result extends DB_result_base {

	// Get the number of rows in the result
	function numRows () {
		return $this->_rr->RecordCount();
	}

	function fetchRow () {
		return $this->_rr->FetchRow();
	}

	function insertedId ($table_name='') {
		return $this->_dbh->Insert_ID();
	}

	function affectedRows () {
		return $this->_dbh->Affected_Rows();
	}
	
}

